#!/usr/bin/php -q
<?php

require_once '/usr/local/opmon/etc/config.php';
require_once '/usr/local/opmon/libexec/php-nagios-plugins/Nagios_Plugin.php';

define("VERSION", "1.0");

$np = null;

function set_nagios() {

	# Constructor
	global $np;

	$np = new Nagios_Plugin(
		array (
			'version' => VERSION . " [kb.opservices.com.br]",

			'blurb' => "Developed by:\n\tOpServices\n"
            			. "Author: \nOtavio Honorio - otavio.honorio\@opservices.com.br\n",

			'usage' => "Usage:\n\t%s -G <hostgroup name> -w <warning> -c <critical>\n"
		)
	);

	$np->add_arg("hostgroup|G=s", "Name of hostgroup in OpMon", 1);
	$np->add_arg("warning|w=i", "Warning value");
	$np->add_arg("critical|c=i", "Critical value");

	$np->getopts();

}

function valid_hostgroup($hostgroup) {

	$query = "select count(*) as hostgroup from opcfg.nagios_hostgroups"
		. " where hostgroup_name = '".$hostgroup."'";

	$sth = do_query($query, "opcfg");

        $result[] = $sth->fields;

	if ($result[0]['hostgroup'] == 0) {
		return false;
	} else {
		return true;
	}
}

function get_hostgroup($hostgroup) {

	$query = "select sum(case when last_hard_state = 0 then 1 else 0 end) as UP,"
		. " sum(case when last_hard_state = 1 then 1 else 0 end) as DOWN"
		. " from opmon4.host_status where host_id in (select host_id"
		. " from opcfg.nagios_hostgroup_membership m join opcfg.nagios_hostgroups g"
		. " using (hostgroup_id) where g.hostgroup_name = '".$hostgroup."')";

	$sth = do_query($query, "opmon4");

	$result[] = $sth->fields;

	return $result;

}

function main() {

	global $np;

	set_nagios();

	$exist = valid_hostgroup($np->opts['hostgroup']);

	if ($exist == false) {
		$np->nagios_exit(
                        1,
                        "hostgroup {$np->opts['hostgroup']} not found."
                );
	}

	$hostgroup = get_hostgroup($np->opts['hostgroup']);

	$up = $hostgroup[0]['UP'];
	$down = $hostgroup[0]['DOWN'];
	$total = $up + $down;

	if (!$total) {
		$np->nagios_exit(
                	1,
	                "hostgroup {$np->opts['hostgroup']} without hosts. Please add hosts to the group."
        	);
	}

	$up_perc = round(($up * 100) / $total, 2);
	$down_perc = 100 - $up_perc;

	$warn = ($np->opts['warning'] * $total) / 100;
	$crit = ($np->opts['critical'] * $total) / 100;
	
	$threshold_down = new Nagios_Plugin_Threshold($warn, $crit, 0, $total);
	$threshold_up = new Nagios_Plugin_Threshold($crit, $warn, 0, $total);
	$threshold_down_perc = new Nagios_Plugin_Threshold($np->opts['warning'], $np->opts['critical'], 0, 100);
	$threshold_up_perc = new Nagios_Plugin_Threshold($np->opts['critical'], $np->opts['warning'], 0, 100);

	$np->add_perfdata("up", $up, '', $threshold_up);
	$np->add_perfdata("down", $down, '', $threshold_down);
	$np->add_perfdata("perc_up", $up_perc, '%', $threshold_up_perc);
	$np->add_perfdata("perc_down", $down_perc, '%', $threshold_down_perc);

	$np->nagios_exit(
		$np->check_threshold($down_perc, $np->opts['warning'], $np->opts['critical']),
		"{$np->opts['hostgroup']}: UP: ({$up}/{$up_perc}%) DOWN: ({$down}/{$down_perc}%)"
	);

}

main();

?>
