<?php

class SNMPManager
{

    private $mapper;

    private $oids = array(
        "processes"            => ".1.3.6.1.2.1.25.4.2.1.2",
        "storage"              => ".1.3.6.1.2.1.25.2.3.1.3",
        "ifName"               => ".1.3.6.1.2.1.31.1.1.1.1",
        "ifSpeed"              => ".1.3.6.1.2.1.2.2.1.5",
        "ifDescr"              => ".1.3.6.1.2.1.2.2.1.2",
        "ifAlias"              => ".1.3.6.1.2.1.31.1.1.1.18",
    );

    public function __construct()
    {
        $this->mapper = new \Models\WebServices\Mapper\Mapper();
        include '/usr/local/opmon/etc/config.php';
    }

    private function getCommunity($data)
    {
        $snmpCommunity = new SNMPCommunity($data->globalauthid);
        return  new SNMPIntegration($snmpCommunity, $data->globalhostaddress . ":" . $data->globalport);
    }

    public function getStorage($data)
    {
        $data->oid = $this->oids['storage'];
        $snmpIntegration = $this->getCommunity($data);
        $scanResult = $snmpIntegration->bulkwalk($data->oid);
        $result = array();

        foreach ($scanResult as $key => $value) {
            $obj = new StdClass();
            $obj->value = $obj->key = $value;
            $result[] = $obj;
            $this->mapper->addElement(
                $value,
                $value,
                array(
                    'storages' => $value
                )
            );
        }

        $this->mapper->setComboData($result);
        return $this->mapper->getOutput();
    }

    public function getProcess($data)
    {
        $data->oid = $this->oids['processes'];
        $snmpIntegration = $this->getCommunity($data);
        $scanResult = $snmpIntegration->bulkwalk($data->oid);
        $result = array();

        foreach ($scanResult as $key => $value) {
            $value = preg_replace('/(["])/', ${1}, $value);
            $obj = new StdClass();
            $obj->value = $obj->key = $value;
            $result[] = $obj;
            $this->mapper->addElement(
                "Process_{$value}",
                $value,
                array(
                    'process' => $value
                )
            );
        }

        $this->mapper->setComboData($result);
        return $this->mapper->getOutput();
    }

    public function getInterfaces($data)
    {
        $data->oid = ($data->localmethod) ? $this->oids[$data->localmethod] : '.1.3.6.1.2.1.31.1.1.1.1';
        $snmpIntegration = $this->getCommunity($data);
        $scanResult = $snmpIntegration->bulkwalk($data->oid);
        $result = array();

        foreach ($scanResult as $key => $value) {
            $obj = new StdClass();
            $obj->value = $obj->key = $value;
            $result[] = $obj;
            $this->mapper->addElement(
                "Inteface_{$value}",
                $value,
                array(
                    'interface' => $value,
                    'ifSpeed' => ($this->getIfSpeed($key, $snmpIntegration))?:0
                )
            );
        }

        $this->mapper->setComboData($result);
        return $this->mapper->getOutput();
    }

    public function getIfSpeed($oid, $snmpIntegration)
    {
        preg_match("/^.+\.(\d+)$/", $oid, $matches);

        if (count($matches) != 2) {
            return false;
        }

        $if_index = $matches[1];
        $oid = $this->oids['ifSpeed'] . ".$if_index";
        $ifSpeed =  $snmpIntegration->bulkwalk($oid);
        return $ifSpeed[$oid] /= 1000;
    }

    public function updateCommand($version)
    {
        if ($version > '1.0-1') {
            return false;
        }

        $command_id = get_command_id_by_name("system-model-snmp-linux-interface");

        $sql = "
           SELECT
               ns.service_id,
               nc.command_id,
               group_concat(ncp.parameter) AS parameter
            FROM nagios_services AS ns
                JOIN nagios_commands AS nc ON nc.command_id = ns.check_command
                JOIN nagios_services_check_command_parameters AS ncp ON ncp.service_id = ns.service_id
            WHERE nc.command_id={$command_id}
            GROUP BY ns.service_id,
                 nc.command_id;
            ";
        $services = do_query($sql, DBOPCFG, OPDB_FETCH_ASSOC)->fetchAll();

        array_walk($services, function (&$service) {
            $service['parameter'] = explode(",", $service['parameter']);
        });

        $sql = 'UPDATE nagios_commands SET command_line = '.
            '"/usr/local/opmon/libexec/modelgroups/snmp-linux/agents/op-snmp-interface -h '.
            '$HOSTADDRESS$:$ARG1$ -i $ARG2$ -t $ARG3$ -r $ARG4$ -m $ARG5$ -I \'$ARG6$\' -s '.
            '$ARG7$ -w $ARG8$ -c $ARG9$ -W $ARG10$ -C $ARG11$ -x $ARG12$ -b $ARG13$" WHERE command_id = '
            . $command_id;

        do_query($sql, DBOPCFG);

        $configManager = new ConfigManager;

        foreach ($services as $service) {
            $sql = 'DELETE FROM nagios_services_check_command_parameters WHERE service_id = ' . $service['service_id'];
            do_query($sql, DBOPCFG);

            $configManager->addServiceCheckParameter($service['service_id'], $service['parameter'][0]);
            $configManager->addServiceCheckParameter($service['service_id'], $service['parameter'][1]);
            $configManager->addServiceCheckParameter($service['service_id'], '5');
            $configManager->addServiceCheckParameter($service['service_id'], '2');
            $configManager->addServiceCheckParameter($service['service_id'], $service['parameter'][2]);
            $configManager->addServiceCheckParameter($service['service_id'], $service['parameter'][3]);
            $configManager->addServiceCheckParameter($service['service_id'], '0');
            $configManager->addServiceCheckParameter($service['service_id'], '80');
            $configManager->addServiceCheckParameter($service['service_id'], '80');
            $configManager->addServiceCheckParameter($service['service_id'], '10');
            $configManager->addServiceCheckParameter($service['service_id'], '5');
            $configManager->addServiceCheckParameter($service['service_id'], '1');
            $configManager->addServiceCheckParameter($service['service_id'], '64');
        }
    }
}
